@@ -8,7 +8,7 @@
 #
 package Lingua::AtD;
 {
-  $Lingua::AtD::VERSION = '1.113000';
+  $Lingua::AtD::VERSION = '1.121550';
 }
 use strict;
 use warnings;
@@ -17,7 +17,8 @@ use Class::Std;
 use LWP::UserAgent;
 use Lingua::AtD::Results;
 use Lingua::AtD::Scores;
-use Lingua::AtD::Exceptions;
+
+#use Lingua::AtD::Exceptions;
 use URI;
 
 # ABSTRACT: Provides an OO wrapper for After the Deadline grammar and spelling service.
@@ -27,6 +28,9 @@ use URI;
     # Attributes
     my %api_key_of :
       ATTR( :init_arg<api_key> :get<api_key> :default<'Lingua-AtD'> );
+    my %throttle_of :
+      ATTR( :init_arg<throttle> :get<throttle> :set<throttle> :default<1> );
+    my %last_call_of : ATTR( :get<last_call> :default<0> );
     my %service_host_of :
       ATTR( :init_arg<host>    :get<service_host> :default<'service.afterthedeadline.com'> );
     my %service_port_of :
@@ -42,6 +46,10 @@ use URI;
           . $service_host_of{$ident} . ':'
           . $service_port_of{$ident} . '/';
 
+        # Generate API Key
+        my $rand_hex = join "", map { unpack "H*", chr( rand(256) ) } 1 .. 16;
+        $api_key_of{$ident} = "Lingua-AtD-$rand_hex";
+
         return;
     }
 
@@ -52,12 +60,24 @@ use URI;
         my $ua    = LWP::UserAgent->new();
         $ua->agent( 'Lingua::AtD/' . $Lingua::AtD::VERSION );
 
+        # Throttle Calls. AtD throws a 503 if called too quickly.
+        my $remaining = $throttle_of{$ident} - ( time - $last_call_of{$ident} );
+        sleep($remaining) if ( $remaining > 0 );
+
         my $response = $ua->post( $url, Content => [ %{$arg_ref} ] );
+
+        $last_call_of{$ident} = time;
+
         if ( $response->is_error() ) {
-            Lingua::AtD::HTTPException->throw(
-                http_status => $response->status_line,
-                service_url => $url,
-            );
+
+            # TODO: Implement Exceptions
+            my $msg = "'$url' responded with '" . $response->status_line . "'.";
+            croak $msg;
+
+            #            Lingua::AtD::HTTPException->throw(
+            #                http_status => $response->status_line,
+            #                service_url => $url,
+            #            );
         }
 
         return $response->content;
@@ -101,41 +121,42 @@ Lingua::AtD - Provides an OO wrapper for After the Deadline grammar and spelling
 
 =head1 VERSION
 
-version 1.113000
+version 1.121550
 
 =head1 SYNOPSIS
 
     use Lingua::AtD;
-    
+
     # Create a new service proxy
     my $atd = Lingua::AtD->new( {
-        host => 'service.afterthedeadline.com',
-        port => 80
+        host     => 'service.afterthedeadline.com',
+        port     => 80,
+        throttle => 1,
     });
 
     # Run spelling and grammar checks. Returns a Lingua::AtD::Response object.
-    my $atd_response = $atd->check_document('Text to check.');
+    my $doc_check = $atd->check_document('Text to check.');
     # Loop through reported document errors.
-    foreach my $atd_error ($atd_response->get_errors()) {
+    foreach my $atd_error ($doc_check->get_errors()) {
         # Do something with...
         print "Error string: ", $atd_error->get_string(), "\n";
     }
-    
-    # Run only grammar checks. Essentially the same as 
+
+    # Run only grammar checks. Essentially the same as
     # check_document(), sans spell-check.
-    my $atd_response = $atd->check_grammar('Text to check.');
+    my $grmr_check = $atd->check_grammar('Text to check.');
     # Loop through reported document errors.
-    foreach my $atd_error ($atd_response->get_errors()) {
+    foreach my $atd_error ($grmr_check->get_errors()) {
         # Do something with...
         print "Error string: ", $atd_error->get_string(), "\n";
     }
-    
+
     # Get statistics on a document. Returns a Lingua::AtD::Scores object.
     my $atd_scores = $atd->stats('Text to check.');
     # Loop through reported document errors.
     foreach my $atd_metric ($atd_scores->get_metrics()) {
         # Do something with...
-        print $atd_metric->get_type(), "/", $atd_metric->get_key(), 
+        print $atd_metric->get_type(), "/", $atd_metric->get_key(),
             " = ", $atd_metric->get_value(), "\n";
     }
 
@@ -147,12 +168,13 @@ Lingua::AtD provides an OO-style interface for After the Deadline's grammar and
 
 =head2 new
 
-This constructor takes three arguments, all optional. The sample below shows the defaults.
+This constructor takes four arguments, all optional. The sample below shows the defaults.
 
     $atd = Lingua::AtD->new({
-        api_key => 'Lingua-AtD',
-        host    => 'service.afterthedeadline.com',
-        port    => 80
+        api_key  => 'Lingua-AtD',
+        host     => 'service.afterthedeadline.com',
+        port     => 80,
+        throttle => 2,
     });
 
 =over 4
@@ -169,6 +191,10 @@ Host for the AtD service. Defaults to the public host: I<service.afterthedeadlin
 
 Port for the AtD service. Defaults to the standard http port: I<80>. AtD's software is open source, and it's entirely possible to download and set up your own private AtD service. See the L<AtD Project website|http://open.afterthedeadline.com/> for details.
 
+=item throttle
+
+There's no API documentation stating such, but testing has shown that AtD service throws a 503 error if called too quickly. This specifies the number of seconds to wait between calls. The default is 1 and seems to work find.
+
 =back
 
 =head2 get_api_key
@@ -195,6 +221,18 @@ Returns the port of the AtD service.
 
 Returns a formatted URL for the AtD service.
 
+=head2 get_throttle
+
+    $atd->get_throttle();
+
+Returns the number of seconds that must pass between calls to the AtD service.
+
+=head2 set_throttle
+
+    $atd->set_throttle(3);
+
+Sets the number of seconds that must pass between calls to the AtD service.
+
 =head2 check_document
 
     $atd_results = $atd->check_document('Some text stringg in badd nneed of prufreding.');
